/*
 * Copyright (c) 2013, Sangoma Technologies
 * James Zhang <jzhang@sangoma.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "esl.h"
#include "scci_def.h"
#include "scci_log.h"
#include "scci_commander.h"
#include "scci_thread.h"
#include "scci_memory.h"

scci_commander_config_t scci_cmd;

scci_status_t scci_config_commander(char *host, int port, char *user, char *pwd)
{
	if(!host || port==0) {
		return SCCI_EINVAL;
	}
	
	memset(&scci_cmd, 0, sizeof(scci_commander_config_t));
	strncpy(scci_cmd.nsg_host, host, strlen(host));
	if(user) {
		strncpy(scci_cmd.nsg_user, user, strlen(user));
	}

	if(pwd) {
		strncpy(scci_cmd.nsg_auth, pwd, strlen(pwd));
	}
	scci_cmd.nsg_port = port;

	
	scci_cmd.el_logger = scci_el_default_logger;
	return SCCI_SUCCESS;
}

scci_status_t scci_commander_connect()
{
	if(!scci_cmd.nsg_host || scci_cmd.nsg_port==0) {
		SCCI_LOG(scci_cmd, SCCI_LOG_CRIT, "\nNSG's host name or port were not set. Not able to connect. \n\s", "");
		return SCCI_FAIL;
	}
	
	memset(&scci_cmd.commander_handle, 0, sizeof(esl_handle_t));
	if(!esl_connect(&scci_cmd.commander_handle, scci_cmd.nsg_host, scci_cmd.nsg_port, scci_cmd.nsg_user, scci_cmd.nsg_auth) ){
		SCCI_LOG(scci_cmd, SCCI_LOG_INFO, "\nCommander connected! \n\s", "");
		return SCCI_SUCCESS;
	}


	scci_sleep(1000);
												
	return SCCI_FAIL;
}

scci_status_t scci_commander_disconnect()
{
	esl_disconnect(&scci_cmd.commander_handle);
	return SCCI_SUCCESS;
}

static void *launch_command(esl_thread_t *me, void *obj)
{
	char *cmd = obj;
	scci_assert(cmd);	

        esl_send_recv(&scci_cmd.commander_handle, cmd);

        if (scci_cmd.commander_handle.last_sr_event && scci_cmd.commander_handle.last_sr_event->body) {
                printf("%s\n", scci_cmd.commander_handle.last_sr_event->body);
        } else {
                // this is unlikely to happen with api or bgapi (which is hardcoded above) but prefix but may be true for other commands
                printf("%s\n", scci_cmd.commander_handle.last_sr_reply);
        }
}

scci_status_t scci_fire_command(char* in)
{
	char *cmd = NULL;
	scci_assert(in);
	cmd = scci_malloc(129);
	if(!cmd) {
		SCCI_LOG(scci_cmd, SCCI_LOG_CRIT, "\nNot able to allocate command memory \n\s", "");
		return SCCI_FALSE;
	}
	memset(cmd, 0, 129);

	memcpy(cmd, in, strlen(in));
	esl_thread_create_detached(launch_command, cmd);
	return SCCI_SUCCESS;
}

