#include "scci_el_sdk.h"

static void *launch_listener(esl_thread_t *me, void *obj)
{
	scci_el_connection_t *conn = obj;
	scci_assert(conn);	
	esl_listen_threaded( conn->el_host, conn->el_port, conn->dispatcher, 128);
}

int scci_start_connection(scci_el_connection_t *conn)
{
	scci_assert( conn );

	conn->evnt_queue_listening = SCCI_TRUE;

	/* create message queue */
	if( scci_queue_create(&conn->scci_evnt_queue, APP_CB_MSG_QUEUE_SIZE) != SCCI_SUCCESS ) {
		fprintf(stderr, "Failed to create message queue!\n");
		return 0;
	}

	/* start socket listening thread for ESL events */	
	/*
	esl_listen_threaded( conn->el_host, conn->el_port, conn->dispatcher, 128);
	*/

	esl_thread_create_detached(launch_listener, conn);
	return 0;
}


int scci_close_connection(scci_el_connection_t *conn) 
{
	scci_assert( conn );

	conn->evnt_queue_listening = SCCI_FALSE;
	scci_sleep(100);

	if( conn->scci_evnt_queue ) {
		scci_queue_destroy( &conn->scci_evnt_queue );
	}
	
	return 0;
}

int scci_on_event_session_call_originate( scci_el_session_t *session, esl_handle_t *handle )
{
	return 0;
}
int scci_on_event_session_call_creat( scci_el_session_t *session, esl_handle_t *handle )
{
	return 0;
}
int scci_on_event_session_call_answer( scci_el_connection_t *conn, esl_handle_t *handle )
{
	if(handle) {
		if(handle->sock!=0) {
			esl_execute(handle, "answer", NULL, NULL);
			return 0;
		}
	}
	
	return 1;
}

int scci_on_event_session_call_hangup( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}
int scci_on_event_session_call_destroy( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}
int scci_on_event_session_call_execute( scci_el_connection_t *conn, esl_handle_t *handle, const char *app, const char *data)
{
	if(!data || !app) {
		return 1;
	}
	if(handle) {
		if(handle->sock!=0) {
			esl_execute(handle, app, data, NULL);
		}
	}
	return 0;
}


int scci_on_event_session_call_hold( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}
int scci_on_event_session_call_unhold( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}
int scci_on_event_session_call_bridge( scci_el_connection_t *conn, esl_handle_t *handle, const char *dest)
{
	if(!dest) {
		return 1;
	}
	if(handle) {
		if(handle->sock!=0) {
			esl_execute(handle, "bridge", dest, NULL);
		}
	}
	return 0;
}
int scci_on_event_session_call_unbridge( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}

int scci_on_event_session_call_application( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}
int scci_on_event_session_call_alarm( scci_el_session_t *session, esl_handle_t *handle  )
{
	return 0;
}


