#include "scci_listener.h"
#include "scci_helper.h"
#include "scci_log.h"
#include "scci_el_sdk.h" 
#include "scci_memory.h"

extern int esl_log_level;

scci_el_connection_t scci_conn;


void scci_init_connection_data(scci_el_dispatch_callback_t disp, scci_queue_t *queue, scci_el_connection_t *conn)
{
	scci_assert(conn);
	scci_assert(queue);
	if (conn) {
		conn->dispatcher = disp;
		conn->el_logger = scci_el_default_logger;

		conn->conn_cb.originate_handler = scci_on_event_session_call_originate;
		conn->conn_cb.create_handler	= scci_on_event_session_call_creat;
		conn->conn_cb.answer_handler	= scci_on_event_session_call_answer;
		conn->conn_cb.hangup_handler	= scci_on_event_session_call_hangup;
		conn->conn_cb.destroy_handler	= scci_on_event_session_call_destroy;
		conn->conn_cb.execute_handler	= scci_on_event_session_call_execute;
		conn->conn_cb.hold_handler		= scci_on_event_session_call_hold;
		conn->conn_cb.unhold_handler	= scci_on_event_session_call_unhold;
		conn->conn_cb.bridge_handler	= scci_on_event_session_call_bridge;
		conn->conn_cb.unbridge_handler	= scci_on_event_session_call_unbridge;
		conn->conn_cb.application_handler	= scci_on_event_session_call_application;
		conn->conn_cb.log_handler		= scci_el_default_logger;
		conn->conn_cb.alarm_handler 	= scci_on_event_session_call_alarm;
	}
	if (queue) {
		conn->app_evnt_queue = queue;
	}
	
	/* create scci receiving message queue */
	if( scci_queue_create(&conn->scci_evnt_queue, APP_CB_MSG_QUEUE_SIZE ) != SCCI_SUCCESS ) {
		conn->evnt_queue_listening = SCCI_FALSE;
		fprintf(stderr, "Failed to create message queue!\n");
	}
}

void scci_el_default_dispatch_cb(esl_socket_t server_sock, esl_socket_t client_sock, struct sockaddr_in *addr) 
{
	int done = 0;
	time_t exp = 0;
	esl_status_t status = ESL_FAIL;

	scci_listener_event_t *evnt = NULL;
	scci_app_cb_event_t *app_cb_evnt = NULL;
	esl_handle_t handle = {{0}};


	char uuid[256] = "";

	esl_attach_handle (&handle, client_sock, addr);
	SCCI_LOG(scci_conn, SCCI_LOG_INFO, "\n\nConnected! %d\n", handle.sock);


#if 0
	int n = 0;
	for( n=0; n<65535; n++) 
	{ 
		if (handle.socket_buf[n]=='\0')
			printf("\\0");
		else if (handle.socket_buf[n]=='\n')
			printf("\\n");
		else if (handle.socket_buf[n]=='\r')
			printf("\\r");
		else if (handle.socket_buf[n]=='\t')
			printf("\\t");
		else
			printf("%c", handle.socket_buf[n]);
	}
#endif

	if( scci_conn.app_evnt_queue ) {
		evnt = scci_malloc(sizeof(scci_listener_event_t));
		scci_assert(evnt);
		evnt->conn = &scci_conn;
		evnt->handle = &handle;
		memset(evnt->evnt_buf, 0, 65535);
		memcpy(evnt->evnt_buf, handle.socket_buf, 65535);
		scci_queue_enqueue( scci_conn.app_evnt_queue, evnt );
		SCCI_LOG(scci_conn, SCCI_LOG_INFO, "queuing event from listener thread %s\n", " ");
	}

	esl_filter(&handle, "unique-id", esl_event_get_header(handle.info_event,"caller-unique-id"));
	strcpy(uuid,esl_event_get_header(handle.info_event, "caller-unique-id"));
	printf("handle.sock=%d,UUID=%s\n",handle.sock,uuid);
	esl_events(&handle, ESL_EVENT_TYPE_PLAIN, "SESSION_HEARTBEAT CHANNEL_ANSWER CHANNEL_ORIGINATE CHANNEL_PROGRESS CHANNEL_HANGUP CHANNEL_BRIDGE CHANNEL_UNBRIDGE CHANNEL_OUTGOING CHANNEL_EXECUTE CHANNEL_EXECUTE_COMPLETE DTMF CUSTOM conference::maintenance");
	// esl_events(&handle, ESL_EVENT_TYPE_PLAIN, "DTMF");

	while((status = esl_recv_timed(&handle, 10)) != ESL_FAIL 
		&& scci_conn.evnt_queue_listening != SCCI_FALSE) {
		if (done) {
			if (time(NULL) >= exp) {
				break;
			}
			break;
		} else if (status == ESL_SUCCESS) {
			const char *type;

			type = esl_event_get_header(handle.last_event, "content-type");

			if (type && !strcasecmp(type, "text/disconnect-notice")) {

				const char *dispo = esl_event_get_header(handle.last_event, "content-disposition");
				SCCI_LOG(scci_conn, SCCI_LOG_INFO, "\n\nGot a disconnection notice dispostion: [%s]\n", dispo ? dispo : "");

				if (!strcmp(dispo, "linger")) {
					done = 1;
					SCCI_LOG(scci_conn, SCCI_LOG_INFO, "Waiting 5 seconds for any remaining events. %s \n", " ");
					exp = time(NULL) + 5;
				}
			} else {
				int res = 0;
				char *evnt_bdy = esl_event_get_body(handle.last_event);

				if(!strcasecmp(type, "text/event-plain")) {
					/* SCCI_LOG(scci_conn, SCCI_LOG_INFO, "evnt_bdy= %s \n", evnt_bdy); */
					if(!strncmp(evnt_bdy,"Event-Name: DTMF",strlen("Event-Name: DTMF"))) {
						SCCI_LOG(scci_conn, SCCI_LOG_INFO, "DTMF event received. %s\n", " " );

						if( scci_conn.app_evnt_queue ) {
							evnt = scci_malloc(sizeof(scci_listener_event_t));
							scci_assert(evnt);
							evnt->conn = &scci_conn;
							evnt->handle = &handle;
							memset(evnt->evnt_buf, 0, 65535);
							memcpy(evnt->evnt_buf, evnt_bdy, strlen(evnt_bdy));
							scci_queue_enqueue( scci_conn.app_evnt_queue, evnt );
							SCCI_LOG(scci_conn, SCCI_LOG_INFO, "queuing event from listener thread %s\n", " ");
						}

					}
				}
			}

#if 0 
			const char *name;
			name = esl_event_get_header(handle.last_event, "Event-Name");
			if (name) {
				scci_el_session_t *s = NULL;
				if (!strcasecmp(name, "channel-data")) {
					if ( "new channel" ) {
						scci_conn.conn_cb.create_handler(scci_conn, handle);
						SCCI_LOG(scci_conn, SCCI_LOG_ERROR, "create new channel %s \n", " " );
					} else if ("hangup") {
						scci_conn.conn_cb.hangup_handler(scci_conn, handle);
						SCCI_LOG(scci_conn, SCCI_LOG_INFO, "hangup channel %s \n", " " );
					} else if ("hold") {
						scci_conn.conn_cb.hold_handler(scci_conn, handle);
						SCCI_LOG(scci_conn, SCCI_LOG_INFO, "hold channel %s \n", " " );
					} else if ("unhold") {
						scci_conn.conn_cb.unhold_handler(scci_conn, handle);
						SCCI_LOG(scci_conn, SCCI_LOG_INFO, "unhold channel %s \n", " " );
					} else  {
						;
					}
				} else if (!strcasecmp(name, "socket_data")) {
					;
				}
			}
#endif
		}

		/* listen to the application callback events */
		if( scci_conn.evnt_queue_listening != SCCI_FALSE ) {
			// SCCI_LOG(scci_conn, SCCI_LOG_INFO, " dequeuing app cb events %s \n", " " );
			app_cb_evnt = scci_queue_dequeue(scci_conn.scci_evnt_queue);
			if(app_cb_evnt) {
				scci_app_cb_event_handler(app_cb_evnt);
				scci_safe_free(app_cb_evnt);
			}
		}
	}

	SCCI_LOG(scci_conn, ESL_LOG_LEVEL_INFO, "Disconnected! %d\n", handle.sock);
	esl_disconnect(&handle);
}

scci_status_t scci_app_cb_event_handler( scci_app_cb_event_t *evnt)
{
	if( !evnt || !evnt->conn || !evnt->handle ) {
		return SCCI_FALSE;
	}
	
	SCCI_LOG_P(evnt->conn, SCCI_LOG_INFO, "cb event received %s \n", (char*)evnt->application );
	if(!strcasecmp(evnt->application, "answer")) {
		if( evnt->conn->conn_cb.answer_handler ) {
			evnt->conn->conn_cb.answer_handler(evnt->conn, evnt->handle);
			return SCCI_TRUE;
		}
	} else {
		/* by-default execute command */
		if( evnt->conn->conn_cb.execute_handler ) {
			evnt->conn->conn_cb.execute_handler(evnt->conn, evnt->handle, evnt->application, evnt->data);
			return SCCI_TRUE;
		}
	}

	return SCCI_FALSE;
}

