/*
 * Copyright (c) 2013, Sangoma Technologies
 * James Zhang <jzhang@sangoma.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "esl.h"
#include "hash.h"
#include "scci_queue.h"
#include "scci_log.h"

#ifndef _SCCI_LISTENER_H_
#define _SCCI_LISTENER_H_

struct scci_event_listener_session;
struct scci_el_connection;
/*********************************
 *
 * event callback function pointer definitions
 *
 * *******************************/
typedef int (*scci_el_handler_originate_t) (struct scci_event_listener_session *session, esl_handle_t *handle);
typedef int (*scci_el_handler_create_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_hangup_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_destroy_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_hold_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_unhold_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_unbridge_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_application_t)(struct scci_event_listener_session *session, esl_handle_t *handle );
typedef int (*scci_el_handler_disconnect_t)(struct scci_event_listener_session *session, esl_handle_t *handle );

typedef int (*scci_el_handler_answer_t)(struct scci_el_connection *conn, esl_handle_t *handle);
typedef int (*scci_el_handler_bridge_t)( struct scci_el_connection *conn, esl_handle_t *handle, const char *dest);
typedef int (*scci_el_handler_execute_t)(struct scci_el_connection *conn, esl_handle_t *handle, const char *app,  const char *data);

typedef int (*scci_el_handler_alarm_t)(struct scci_event_listener_session *session, esl_handle_t *handle );

typedef void (*scci_el_dispatch_callback_t)(esl_socket_t server_sock, esl_socket_t client_sock, struct sockaddr_in *addr);



/*********************************
 *
 * data structures
 *
 * *******************************/
typedef struct scci_app_callback_handler {
    scci_el_handler_originate_t     originate_handler;
    scci_el_handler_create_t        create_handler;
    scci_el_handler_answer_t        answer_handler;
    scci_el_handler_hangup_t        hangup_handler;
    scci_el_handler_destroy_t       destroy_handler;
    scci_el_handler_execute_t       execute_handler;
    scci_el_handler_hold_t          hold_handler;
    scci_el_handler_unhold_t        unhold_handler;
    scci_el_handler_bridge_t        bridge_handler;
    scci_el_handler_unbridge_t      unbridge_handler;
    scci_el_handler_application_t   application_handler;
    scci_el_handler_log_t           log_handler;
    scci_el_handler_alarm_t         alarm_handler;
} scci_app_callback_handler_t;

typedef struct scci_event_listener_connection{
    char el_host[256];
    esl_port_t el_port;
    char el_auth[256];

    scci_el_handler_log_t   el_logger;
    struct scci_app_callback_handler conn_cb;
    scci_el_dispatch_callback_t dispatcher;

	scci_queue_t *app_evnt_queue;
	scci_queue_t *scci_evnt_queue;
	scci_bool_t evnt_queue_listening;
}scci_el_connection_t;

typedef struct scci_listner_event {
	scci_el_connection_t *conn;
	esl_handle_t *handle;
	char evnt_buf[65535];
} scci_listener_event_t;

typedef struct scci_app_cb_event {
	scci_el_connection_t *conn;
	esl_handle_t *handle;
	char application[255];
	char data[2048];
} scci_app_cb_event_t;

typedef struct scci_hash {
    Hash table;
} scci_hash_t;

#define APP_CB_MSG_QUEUE_SIZE 200
#endif
