
#ifdef WIN32
#   if (_WIN32_WINNT < 0x0400)
#       error "Need to target at least Windows 95/WINNT 4.0 because TryEnterCriticalSection is needed"
#   endif
#   include <windows.h>
#endif


#include "scci_thread.h"
#include "scci_memory.h"


#ifdef WIN32
#include <process.h>

#define SCCI_THREAD_CALLING_CONVENTION __stdcall

struct scci_mutex {
	CRITICAL_SECTION mutex;
};

#else
#include <pthread.h>
#include <poll.h>
#include <errno.h>

#define SCCI_THREAD_CALLING_CONVENTION

#ifndef PTHREAD_MUTEX_RECURSIVE
#define PTHREAD_MUTEX_RECURSIVE PTHREAD_MUTEX_RECURSIVE_NP
#endif /* PTHREAD_MUTEX_RECURSIVE */
				  
#endif

struct scci_thread 
{
#ifdef WIN32
		void *handle;
#else
		pthread_t handle;
#endif
		void *private_data;
		scci_thread_function_t function;
		scci_size_t stack_size;
#ifndef WIN32
		pthread_attr_t attribute;
#endif
};

scci_size_t scci_thread_default_stacksize = 240 * 1024;


static void * SCCI_THREAD_CALLING_CONVENTION thread_launch(void *args)
{
	void *exit_val;
	scci_thread_t *thread = (scci_thread_t *)args;
	exit_val = thread->function(thread, thread->private_data);
#ifndef WIN32
	pthread_attr_destroy(&thread->attribute);
#endif
	scci_safe_free(thread);

	return exit_val;
}


SCCI_DECLARE(scci_status_t) scci_thread_create_detached(scci_thread_function_t func, void *data)
{
	return scci_thread_create_detached_ex(func, data, scci_thread_default_stacksize);
}

SCCI_DECLARE(scci_status_t) scci_thread_create_detached_ex(scci_thread_function_t func, void *data, scci_size_t stack_size)
{
	scci_thread_t *thread = NULL;
	scci_status_t status = SCCI_FAIL;

	if (!func || !(thread = (scci_thread_t *)scci_calloc(1, sizeof(scci_thread_t)))) {
		goto done;
	}

	thread->private_data = data;
	thread->function = func;
	thread->stack_size = stack_size;

#if defined(WIN32)
	thread->handle = (void *)_beginthreadex(NULL, (unsigned)thread->stack_size, (unsigned int (__stdcall *)(void *))thread_launch, thread, 0, NULL);
	if (!thread->handle) {
		goto fail;
	}
	CloseHandle(thread->handle);

	status = SCCI_SUCCESS;
	goto done;
#else
	
	if (pthread_attr_init(&thread->attribute) != 0)	goto fail;

	if (pthread_attr_setdetachstate(&thread->attribute, PTHREAD_CREATE_DETACHED) != 0) goto failpthread;

	if (thread->stack_size && pthread_attr_setstacksize(&thread->attribute, thread->stack_size) != 0) goto failpthread;

	if (pthread_create(&thread->handle, &thread->attribute, thread_launch, thread) != 0) goto failpthread;

	status = SCCI_SUCCESS;
	goto done;
 failpthread:
	pthread_attr_destroy(&thread->attribute);
#endif

 fail:
	if (thread) {
		scci_safe_free(thread);
	}
 done:
	return status;
}

