/*
 *
 * James Zhang <jzhang@sangoma.com>
 * Copyright (C) 2011, Sangoma Technologies.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *
 */
#ifndef _SCCI_THREAD_H
#define _SCCI_THREAD_H


#include <stdlib.h>
#include <stdint.h>

#include "scci_thread_declare.h"


typedef size_t scci_size_t;

typedef enum {
	SCCI_LOG_CRIT,
	SCCI_LOG_ERROR,
	SCCI_LOG_WARNING,
	SCCI_LOG_INFO,
	SCCI_LOG_NOTICE,
	SCCI_LOG_DEBUG,
} scci_loglevel_t;

/*! \brief sleep x amount of milliseconds */
#ifdef __WINDOWS__
#define scci_sleep(x) Sleep(x)
#else
#define scci_sleep(x) usleep(x * 1000)
#endif

typedef struct scci_thread scci_thread_t;
typedef void *(*scci_thread_function_t) (scci_thread_t *, void *);

SCCI_DECLARE(scci_status_t) scci_thread_create_detached(scci_thread_function_t func, void *data);
SCCI_DECLARE(scci_status_t) scci_thread_create_detached_ex(scci_thread_function_t func, void *data, scci_size_t stack_size);


typedef void (*scci_log_func_t)(uint8_t level, char *fmt, ...);
typedef void* (*scci_calloc_func_t)(scci_size_t nmemb, scci_size_t size);	
typedef void (*scci_free_func_t)(void *ptr);
typedef void (*scci_assert_func_t)(char *message);

#endif

/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
 */

/******************************************************************************/


