/*
 * Copyright (c) 2013, Sangoma Technologies
 * James Zhang <jzhang@sangoma.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "scci_el_sdk.h"
#include "esl.h"

#include "scci_thread.h"
#include "scci_queue.h"
#include "scci_memory.h"
#include "scci_listener.h"


extern scci_el_connection_t scci_conn;


#define IVR_MSG_QUEUE_SIZE 200
scci_queue_t *ivr_msg_queue;

int g_ivr_running = 1;

static int sample_handle_raw_event(scci_listener_event_t *evnt)
{
	int len = 0;
	int n = 0;
	char *p = NULL;
	char *p_key = NULL;
	char *p_value = NULL;

	for( p=evnt->evnt_buf,n=0; p&&n<65535; ) 
	{
		len = strlen(p);

		if( *p != '\0' ) {
			p_key = p;

			p += len +1;
			n += len +1;
			if (*p == 0) {
				break;
			}
			len = strlen(p);
			p_value = p;
			
			if (!strcasecmp(p_key, "FreeSWITCH-IPv6")) {
				p += len +1;
				n += len +1;
				if (*p == '\0') {
					break;
				}
				len = strlen(p);
				p_value = p;
			} else if ( !strcasecmp(p_key, "Event-Date-Local") 
						|| !strcasecmp(p_key, "Event-Date-GMT") 
						|| !strcasecmp(p_key, "Channel-Channel-Name")
						|| !strcasecmp(p_key, "Channel-Name")
						|| !strcasecmp(p_key, "Caller-Channel-Name")
						|| !strcasecmp(p_key, "variable_sip_from_uri")
						|| !strcasecmp(p_key, "variable_sip_full_via")
						|| !strcasecmp(p_key, "variable_sip_full_from")
						|| !strcasecmp(p_key, "variable_sip_full_to")
						|| !strcasecmp(p_key, "variable_sip_req_uri")
						|| !strcasecmp(p_key, "variable_sip_to_uri")
						|| !strcasecmp(p_key, "variable_sip_contact_uri")
						|| !strcasecmp(p_key, "variable_channel_name")
						|| !strcasecmp(p_key, "variable_sip_user_agent")
						|| !strcasecmp(p_key, "variable_switch_r_sdp")
						|| !strcasecmp(p_key, "variable_DP_MATCH")
						|| !strcasecmp(p_key, "variable_current_application_data")
						|| !strcasecmp(p_key, "Reply-Text")
			) {
				p += len +1;
				n += len +1;
				if (*p == '\0') {
					break;
				}
				len = strlen(p);
				
			} else if ( !strcasecmp(p_key, "content-value")) {
				p += len+1;
				n += len+1;
				// if double \0\0, that means end of event socket
				if (*p == 0) {
					break;
				}
			}
			scci_el_default_logger(__FILE__, __FUNCTION__, __LINE__, SCCI_LOG_INFO, "p_key=%s  p_value = %s \n", p_key, p_value);
		}
	}
	
	return 0;
}

static int sample_handle_dtmf(scci_listener_event_t *evnt)
{
	scci_assert(evnt);
	scci_assert(evnt->evnt_buf);

	scci_el_default_logger(__FILE__, __FUNCTION__, __LINE__, SCCI_LOG_INFO, "conn->sock_id = %d \n", evnt->handle->sock);
	
	if(!strncmp(evnt->evnt_buf,"Event-Name: DTMF",strlen("Event-Name: DTMF"))) {
		/*
		printf("\n\nDTMF RECEIVED : %s\n\n", evnt->evnt_buf);
		*/
		char *p_digit = strstr(evnt->evnt_buf, "DTMF-Digit: ");
		char *p_duration = strstr(evnt->evnt_buf, "DTMF-Duration:");
		char value[3];
		int is_sharp = 0;

		scci_assert(p_digit);
		scci_assert(p_duration);
		scci_assert(p_digit+12);
		p_digit += 12;

		memset(value, 0, 3);
		if((p_duration-p_digit)>3) {
			is_sharp = 1;
		}
		memcpy(value, p_digit, (is_sharp)?3:1);
		
		if(is_sharp) {
			is_sharp = !strncmp(value, "%23", 3);
			// double check 
		}
		if(is_sharp) {
			memset(value, 0, 3);
			value[0] = '#';
		} 
		switch(value[0]) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		case '*':
		case '#':
			printf("\nDTMF received: %c\n", value[0]);
			break;
		default:
			printf("\nInvalid DTMF key received: %c\n", value[0]);
			break;
		}

		return 0;
	}
	
	return 1;
}

static int sample_play(scci_listener_event_t *evnt)
{
	scci_app_cb_event_t *evnt_ply = scci_malloc(sizeof(scci_app_cb_event_t)) ;
	scci_assert(evnt_ply);
	scci_assert(evnt);

	memset(evnt_ply, 0, sizeof(scci_app_cb_event_t));
	evnt_ply->conn = evnt->conn;
	evnt_ply->handle = evnt->handle;
	memcpy(evnt_ply->application, "playback", 8);
	memcpy(evnt_ply->data, "/tmp/rec/rec.wav", 16);
	if( evnt_ply ) {
		scci_queue_enqueue(evnt->conn->scci_evnt_queue, (void*)evnt_ply);
	}
	return 0;
}

static int sample_answer(scci_listener_event_t *evnt)
{
	scci_app_cb_event_t *evnt_ans = scci_malloc(sizeof(scci_app_cb_event_t)) ;

	scci_assert(evnt_ans);
	scci_assert(evnt);

	memset(evnt_ans, 0, sizeof(scci_app_cb_event_t));
	evnt_ans->conn = evnt->conn;
	evnt_ans->handle = evnt->handle;
	memcpy(evnt_ans->application, "answer", 6);
	if( evnt_ans ) {
		scci_queue_enqueue(evnt->conn->scci_evnt_queue, (void*)evnt_ans);
	}
	return 0;
}

static int sample_record(scci_listener_event_t *evnt)
{
	scci_app_cb_event_t *evnt_rec = scci_malloc(sizeof(scci_app_cb_event_t)) ;
	scci_assert(evnt_rec);
	scci_assert(evnt);

	memset(evnt_rec, 0, sizeof(scci_app_cb_event_t));
	evnt_rec->conn = evnt->conn;
	evnt_rec->handle = evnt->handle;
	memcpy(evnt_rec->application, "record_session", 14);
	memcpy(evnt_rec->data, "/tmp/rec.wav", 12);
	if( evnt_rec ) {
		scci_queue_enqueue(evnt->conn->scci_evnt_queue, (void*)evnt_rec);
	}
	return 0;
}

static void *app_ivr_event_queue_handler(scci_thread_t *me, void *obj)
{
	scci_listener_event_t *evnt = NULL;

	while( g_ivr_running ) {
		evnt = scci_queue_dequeue(ivr_msg_queue);
		if (!evnt)
		{
			scci_sleep(100);
			continue;
		}

		if( sample_handle_dtmf( evnt ) ) {
			sample_answer(evnt);
			sample_record(evnt);
			/*
			sample_play(evnt);
			*/
		}

		scci_safe_free(evnt);
	}
	return NULL;
}

void call_sip(char *sip_profile, char *sip_addr)
{
	char cmd[128];
	sprintf(cmd, "api originate sofia/%s/%s &echo", sip_profile, sip_addr);
	scci_fire_command(cmd);
}

void call_isup(char *called_no, char *calling_no, int span_no, int chan_no)
{
	char cmd[128];
	sprintf(cmd, "api originate freetdm/%d/%d/%s &echo", span_no, chan_no, called_no);
	scci_fire_command(cmd);
}

void sample_app_call_sip()
{
	char sip_addr[129] = "";
	char sip_profile[129] = "";
	char input[129] = "";

enter_profile:
    	fprintf(stdout, "Please enter the sip profile name:");
	if (fgets(input, 128, stdin) == NULL) {
		fprintf(stdout, "\nError try again");
		goto enter_profile;
	} 
	strncat(sip_profile, input, strlen(input)-1);
enter_sipaddr:
    	fprintf(stdout, "Please enter the sip address to call:");
	if (fgets(input, 128, stdin) == NULL) {
		fprintf(stdout, "\nError try again");
		goto enter_sipaddr;
	} 
	strncat(sip_addr, input, strlen(input)-1);

	call_sip(sip_profile, sip_addr);
}

void sample_app_call_isup()
{
	char called_no[30] = "";
	char calling_no[30] = "";
	char destination[30] = "";
	int span_no = 2;
	int chan_no = 3;

	char str_span[10];
	char str_chan[10];
	memset(str_span, 0, 10);
	memset(str_chan, 0, 10);
	
enter_span:
	fprintf(stdout, "Enter span number:");
    	if (!fgets(str_span, 9, stdin)) {
		fprintf(stdout, "Invalid input.");
		goto enter_span;	
	}
	span_no = atoi(str_span);
	if(!span_no){
		fprintf(stdout, "Invalid input.");
		goto enter_span;	
	}


enter_chan:
	fprintf(stdout, "\nEnter chan number:");
    	if (!fgets(str_chan, 9, stdin)) {
		fprintf(stdout, "Invalid input.");
		goto enter_chan;
	}
	chan_no = atoi(str_chan);
	if(!chan_no){
		fprintf(stdout, "Invalid input.");
		goto enter_chan;	
	}

enter_called:
	fprintf(stdout, "\nEnter the phone number to call:");
    	if (!fgets(called_no, 29, stdin)) {
		fprintf(stdout, "Invalid input.");
		goto enter_called;
	}
	strncat(destination, called_no, strlen(called_no)-1);
	
	call_isup(destination, calling_no, span_no, chan_no);
}

int sample_main_menu(void)
{
    char choice[6];

    fprintf(stdout, "\n\n\n=========================================================\n");
    fprintf(stdout, "       WELCOME TO SANGOMA SDK SAMPLE APPLICATION\n");
    fprintf(stdout, "=========================================================\n");
    fprintf(stdout, "Please select any one of the below options :\n");

    fprintf(stdout, "Select Action\n");
    fprintf(stdout, "1. Make SIP call \n2. Make ISUP call \n3. Exit\n");
    fprintf(stdout, "Enter your choice[1-3]: ");

    if (NULL != fgets(choice, 6, stdin)) {
        if (2 < strlen(choice)) {
            fprintf(stderr, "Invalid Input Choice must must be [1-4]...\n");
            return 0;
        }

        choice[1] = '\0';

        /* Provides option to user to perform various operations like:
         * 1) Send SMS
         * 2) Print Span related information
         * 3) Exit from application
         */
        switch (choice[0]) {
        case '1':
            sample_app_call_sip();
            break;

        case '2':
            sample_app_call_isup();
            break;

        case '3':
            fprintf(stdout, "SDK Sample Application is getting closed...\n");
            fprintf(stdout, "\n\n\n=========================================================\n");
            fprintf(stdout, "                      GOOD BYE                            \n");
            fprintf(stdout, "=========================================================\n");
            return -1;

        default:
            fprintf(stderr, "Invalid Option. Please Choose a correct option value...\n");
            break;
        }
    }

    return 0;
}


int main(int argc, char **argv) 
{
	/*
	 * config el listener
	 * config el commander
	 * init queue and queue listening thread
	 * register callback
	 * launch queue thread
	 * bridge call in thread based on caller id
	 * hangup call in thread based on destination
	 * play file in thread based on destination
	 */
	

	/* listner's network configuration */
	const char ipaddr[] = "localhost";
	strncpy( scci_conn.el_host, ipaddr, strlen(ipaddr) );
	scci_conn.el_port = 8084;

	/* create message queue */
	if( scci_queue_create(&ivr_msg_queue, IVR_MSG_QUEUE_SIZE) != SCCI_SUCCESS ) {
		fprintf(stderr, "Failed to create message queue!\n");
		return 0;
	}

	/* launch message queue processing thread */
	if (scci_thread_create_detached(app_ivr_event_queue_handler, NULL) != SCCI_SUCCESS) {
		scci_el_default_logger(__FILE__, __FUNCTION__, __LINE__, SCCI_LOG_ERROR, "Failed to create message queue handler thread. \n");
		goto exit_sample_ivr;
	}

	/* set esl logger to 'no output' */
	esl_global_set_default_logger(0);

	/* config el listener & default dispatcher */
	scci_init_connection_data(scci_el_default_dispatch_cb, ivr_msg_queue, &scci_conn);

	/* start connection, and start listening */
	scci_start_connection(&scci_conn);

	scci_config_commander("127.0.0.1", 8821, NULL, "ClueCon");
	scci_commander_connect();
	
    while( !sample_main_menu() ) {
        ;
    }

exit_sample_ivr:

    scci_commander_disconnect();
	
    /* Stop all threads. */
    g_ivr_running = 0;
    scci_sleep(1000);

    /* close connection, stop listening and clear listening queue */
    scci_close_connection(&scci_conn);
    scci_queue_destroy( &ivr_msg_queue );

    return 0;
}

